import java.io.*;
import java.util.*;

final class console {
	static List<String> genBoard(board b, boolean compact) {
		List<String> list = new ArrayList<String>();
		int dimensions = b.getDimensions();
		int x, y;
		String line;

		for(y=dimensions - 1; y>=0; y--)
		{
			line = "  " + (y + 1);
			line = line.substring(line.length() - 2);
			line = " "+ line + " ";

			for(x=0; x<dimensions; x++)
			{
				cross cur_obj = b.getAt(x, y);

				if (cur_obj == cross.empty)
				{
					if (compact)
						line += ".";
					else
						line += "+";
				}
				else if (cur_obj == cross.white)
					line += "o";
				else if (cur_obj == cross.black)
					line += "*";
				else
					line += "?";

				if (!compact && x < (dimensions - 1))
					line += "-";
			}
			list.add(line);

			if (!compact && y > 0)
			{
				line = "   ";

				for(x=0; x<dimensions; x++)
					line += "| ";
				list.add(line);
			}
		}

		line = "    ";
		for(x=0; x<dimensions; x++)
		{
			char x_pos = (char)('a' + x);
			if (x >= 8)
				x_pos++;
			line += "" + x_pos;
			if (!compact)
				line += " ";
		}
		list.add(line);

		return list;
	}

	static void displayBoard(board b, boolean compact)
	{
		List<String> list = genBoard(b, compact);

		for(String current : list)
			System.out.println(current);
	}

	static void loop(int dimensions, boolean random_allowed, double thinkTime, boolean deep, List<Output> outputs) throws InterruptedException
	{
		scene s = new scene(dimensions, false);
		player computer, opponent, current_player = player.black;
		int c;

		System.out.println("stop v" + Version.getVersion() + ", (C) 2009-2015 by folkert@vanheusden.com");
		System.out.println("");

		System.out.println("Play with black or white? (b/w)");
		do
		{
			try
			{
				c = Character.toUpperCase(System.in.read());
			}
			catch(Exception e)
			{
				// System.out.println("Exception: " + e);
				c = -1;
			}
		}
		while(c != 'W' && c != 'B');
		if (c == 'W')
		{
			opponent = player.white;
			computer = player.black;
		}
		else
		{
			opponent = player.black;
			computer = player.white;
		}

		System.out.println("Expecting moves in 'a1...xyy' format, quit/exit, hint or pass.");

		int moveNr = 0;
		for(;;)
		{
			int x = -1, y = -1;
			displayBoard(s.getBoard(), true);
			System.out.println("White points: " + s.getWhitePoints());
			System.out.println("Black points: " + s.getBlackPoints());

			if (current_player == opponent)
			{
				BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

				for(;;)
				{
				hint_back:
					System.out.print("Your move: ");

					try
					{
						String moveString = in.readLine().trim();

						if (moveString.equals("pass"))
						{
							x = y = -1;
							break;
						}
						else if (moveString.equals("hint"))
						{
							SelectedMove selectedMove = s.decideMove(current_player, random_allowed, deep, thinkTime, null, -1);
							if (selectedMove == null)
								System.out.println("Hint: ???");
							else
								System.out.println("Hint: " + Statics.genMoveString(selectedMove.getX(), selectedMove.getY()));
						}
						else if (moveString.equals("quit") || moveString.equals("exit"))
						{
							System.exit(0);
						}
						else
						{
							x = moveString.charAt(0) - 'a';
							if (x < 0)
								x = moveString.charAt(0) - 'A';
							if (x >= 8) // 'i' versus 'j'
								x--;
							y = Integer.valueOf(moveString.substring(1)) - 1;

							if (x<0 || x>=dimensions || y<0 || y>=dimensions)
							{
								System.out.println(moveString + " is an invalid coordinate");
							}
							else if (s.getBoard().getAt(x, y) != cross.empty)
							{
								System.out.println(moveString + " already contains a stone");
							}
							else if (Brain.isValidMove(s.getBoard(), s.getChains(), x, y, (current_player == player.white ? cross.white : cross.black)) == false)
							{
								System.out.println(moveString + " is not a valid move - suicide? KO?");
							}
							else
							{
								break;
							}
						}
					}
					catch(Exception e)
					{
						//						System.out.println("Exception: " + e);
					}
				}
			}
			else
			{
				SelectedMove selectedMove = s.decideMove(current_player, random_allowed, deep, thinkTime, outputs, moveNr++);
				if (selectedMove == null)
				{
					System.out.println("Giving up");
					return;
				}

				x = selectedMove.getX();
				y = selectedMove.getY();

				System.out.println("My move: " + Statics.genMoveString(x, y));
System.out.println("" + selectedMove.dump());
			}

			if (x != -1 && y != -1)
			{
				cross type = cross.empty;

				if (current_player == player.white)
					type = cross.white;
				else
					type = cross.black;

				scene oldScene = s;
				s = s.doMove(x, y, type, current_player);
				oldScene.destructor();
			}
			else
			{
				System.out.println("pass");
			}

			// s.dumpChains(); // FIXME remove me

			if (current_player == player.white)
				current_player = player.black;
			else
				current_player = player.white;
		}
	}
}
